#include <stdexcept>
#include <iostream>
#include <iomanip>
#include <cmath>

#include "srtm.h"
#include "color.h"
#include "map.h"

inline int dms(int d, int m, int s) {return d*3600+m*60+s;}
inline int dms(int d, int m) {return d*3600+m*60;}
inline int dms(int d) {return d*3600;}
                                 //  
//int lat = dms(51,30,00)/3; //    ,   
//int lon = dms(104,1,00)/3;

//int lon = 1200*102 + 327;
//int lat = 1200*52 + 1200 - 1138;
int lon = 1200*115 +658;
int lat = 1200*57 + 1200 - 563;
//int lat = 63973;
//int lon =  121351;

srtm s("/d2/SRTM/hgt/", 10, mode_interp);

std::set<point> points_done;

std::map<point, char> point_cache;


int rmax = 2000; //     .
int dh  = 80;
int dxy = 500;

int safe_geth(point p){
    int h = s.geth(p.x, p.y);
    if (h > srtm_min_interp) h -= srtm_zer_interp;
    if (h < srtm_min) {
       std::cerr << "Can't find SRTM-data for point " 
                 << p.x/1200.0 << " " << p.y/1200.0 << "\n";
       exit(0);
    } 
    return h;
}

std::list<point> L;
std::set<point> B;


double area(point p){

  double ret = pow(6380.0 * M_PI/srtm_width/180, 2)*
      cos(p.x/1200.0 * M_PI/180.0);

  points_done.insert(p); 
  int h0 = safe_geth(p);
//  std::set<point> points_done_tmp;

//  if (h0 > 850) dh=300;
//  else dh=100;


  for (int dir=0; dir<point_adjs; dir++){ //    
    
    //        
    point np = p.adjacent(dir);
    if (points_done.find(np) != points_done.end()) continue;

    // ,      
    L.clear();
    L.push_back(np);
    B = border(L);

    point sp;
    int n;
    for (n = 0; n< rmax; n++){
      int min = -srtm_min;
      for (std::set<point>::iterator b = B.begin(); b!=B.end(); b++){
        int h = safe_geth(*b);
        if (min > h){ min=h; sp=*b;}
      }
      if (sp == p) break;
      if (points_done.find(sp) != points_done.end()) break;
      if (min < h0-dh) break;
      if (sp.x-p.x >dxy) break;
      if (sp.y-p.y >dxy) break;
      add_pb(sp, L,B);
    }

///    std::cerr << n << "\n"; 




//    std::cerr << n <<"\n";
    //        rmax  -   
    if (sp != p)  {/*points_done_l.insert(L.begin(), L.end());*/ continue;}
    //     
    double a = area(np);

    ret+=a;
    if (a > 5){
      std::cout << np.x << " " 
                << np.y << " " 
                << a << " " 
                << dir << " " 
                << safe_geth(np) << "\n";
      std::cerr << np.x << " " 
                << np.y << " " 
                << a << " " 
                << dir << " " 
                << safe_geth(np) << "\n";

    }
  }
  return ret;
}



main(){ 
try {
  point p = point(lat, lon);


  int h0 = safe_geth(p);
  std::cerr << "point1: " 
            << p.x/1200.0 << " "
            << p.y/1200.0 << " "
            << h0 << "\n";


  do {
    int dmin=-1;
    for (int d =0; d<point_adjs; d++){
      int h = safe_geth(p.adjacent(d));
      if (h<h0) {h0=h; dmin=d;}
    }
    if (dmin == -1) break;
    p = p.adjacent(dmin);
  } while (true);

  std::cerr << "point2: " 
            << p.x/1200.0 << " "
            << p.y/1200.0 << " "
            << h0 << "\n";

  area(p);

} catch(std::domain_error e){ std::cerr << e.what(); }
}

